@@ -1,18 +1,16 @@
 
 import logging
 import os
-import time
 from datetime import datetime
 import numbers
 import re
-import math
-import random
 import numpy as np
 import pandas as pd
 from sklearn import model_selection, tree, ensemble, svm, linear_model, neighbors, metrics
 from sklearn.model_selection import GroupKFold, StratifiedKFold
 from joblib import dump, load
-from rdflib import Graph, URIRef, Literal, RDF, ConjunctiveGraph, Namespace
+import pkg_resources
+from openpredict.train_utils import generate_classifier_metadata
 
 def adjcencydict2matrix(df, name1, name2):
     """Convert dict to matrix
@@ -75,7 +73,8 @@ def mergeFeatureMatrix(drugfeatfiles, diseasefeatfiles):
 
 
 def generatePairs(drug_df, disease_df, drugDiseaseKnown):
-    """Generate positive and negative pairs using the Drug dataframe, the Disease dataframe and known drug-disease associations dataframe 
+    """Generate positive and negative pairs using the Drug dataframe, 
+    the Disease dataframe and known drug-disease associations dataframe 
 
     :param drug_df: Drug dataframe
     :param disease_df: Disease dataframe
@@ -171,6 +170,17 @@ def createFeatureArray(drug, disease, knownDrugDisease, drugDFs, diseaseDFs):
 
 
 def sparkBuildFeatures(sc, pairs, classes, knownDrugDis,  drug_df, disease_df):
+    """Create the feature matrix for Spark.
+
+    :param sc: Spark context
+    :param pairs: Generated pairs
+    :param classes: Classes corresponding to the pairs
+    :param knownDrugDisease: Known drug-disease associations
+    :param drugDFs: Drug dataframes
+    :param diseaseDFs: Disease dataframes
+    :return: The features dataframe 
+    """
+
     rdd = sc.parallelize(list(zip(pairs[:,0], pairs[:,1], classes))).map(lambda x: (x[0],x[1],x[2], createFeatureArray( x[0], x[1], knownDrugDis,  drug_df, disease_df)))
     all_scores = rdd.collect()
     drug_col = drug_df.columns.levels[0]
@@ -315,7 +325,7 @@ def evaluate(test_df, clf):
     return scores
 
 
-def get_drug_disease_classifier():
+def train_drug_disease_classifier():
     """The main function to run the drug-disease similarities pipeline, 
     and build the drug-disease classifier.
     It returns, and stores the generated classifier as a `.joblib` file 
@@ -325,22 +335,21 @@ def get_drug_disease_classifier():
     """
     time_start = datetime.now()
     features_folder = "data/features/"
-    resources_folder = "data/resources/"
     drugfeatfiles = ['drugs-fingerprint-sim.csv','drugs-se-sim.csv', 
                      'drugs-ppi-sim.csv', 'drugs-target-go-sim.csv','drugs-target-seq-sim.csv']
     diseasefeatfiles =['diseases-hpo-sim.csv',  'diseases-pheno-sim.csv' ]
-    drugfeatfiles = [ os.path.join(features_folder, fn) for fn in drugfeatfiles]
-    diseasefeatfiles = [ os.path.join(features_folder, fn) for fn in diseasefeatfiles]
+    drugfeatfiles = [ pkg_resources.resource_filename('openpredict', os.path.join(features_folder, fn)) for fn in drugfeatfiles]
+    diseasefeatfiles = [ pkg_resources.resource_filename('openpredict', os.path.join(features_folder, fn)) for fn in diseasefeatfiles]
 
     # Prepare drug-disease dictionary
-    drugDiseaseKnown = pd.read_csv(resources_folder + 'openpredict-omim-drug.csv',delimiter=',') 
+    drugDiseaseKnown = pd.read_csv(pkg_resources.resource_filename('openpredict', 'data/resources/openpredict-omim-drug.csv'),delimiter=',') 
     drugDiseaseKnown.rename(columns={'drugid':'Drug','omimid':'Disease'}, inplace=True)
     drugDiseaseKnown.Disease = drugDiseaseKnown.Disease.astype(str)
     # print(drugDiseaseKnown.head())
 
     # Merge feature matrix
     drug_df, disease_df = mergeFeatureMatrix(drugfeatfiles, diseasefeatfiles)
-    # print(drug_df.head())
+    dump((drug_df, disease_df), 'openpredict/data/features/drug_disease_dataframes.joblib')
 
     # Generate positive and negative pairs
     pairs, classes = generatePairs(drug_df, disease_df, drugDiseaseKnown)
@@ -386,13 +395,40 @@ def get_drug_disease_classifier():
     print("\nTest results 🏆")
     print(scores)
 
+    print("\n Train the final model using all dataset")
+    final_training = datetime.now()
+    train_df = createFeaturesSparkOrDF(pairs, classes, drug_df, disease_df)
+    clf = linear_model.LogisticRegression(penalty='l2', dual=False, tol=0.0001, C=1.0, random_state=n_seed) 
+    clf = trainModel(train_df, clf)
+    print('Final model training runtime 🕕  ' + str(datetime.now() - final_training))
+
     print('\nStore the model in a .joblib file 💾')
-    dump(clf, 'data/models/drug_disease_model.joblib')
+    dump(clf, 'openpredict/data/models/drug_disease_model.joblib')
     # See skikit docs: https://scikit-learn.org/stable/modules/model_persistence.html
 
+    generate_classifier_metadata('openpredict-omim-drugbank-0', scores, "Original OpenPredict classifier based on OMIM and DrugBank")
     print('Complete runtime 🕛  ' + str(datetime.now() - time_start))
     return clf, scores
 
+
+def createFeaturesSparkOrDF(pairs, classes, drug_df, disease_df):
+
+    try:
+        logging.info('Running Spark...')
+        from pyspark import SparkConf, SparkContext
+        sc = SparkContext.getOrCreate()
+        logging.info(sc)
+        drug_df_bc= sc.broadcast(drug_df)
+        disease_df_bc = sc.broadcast(disease_df)
+        knownDrugDis_bc = sc.broadcast(pairs[classes==1])
+        feature_df= sparkBuildFeatures(sc, pairs, classes, knownDrugDis_bc.value,  drug_df_bc.value, disease_df_bc.value)
+
+    except:
+        logging.info("Spark cluster not found. Using pandas to create feature dataframes")
+        feature_df = createFeatureDF(pairs, classes, pairs[classes==1], drug_df, disease_df)
+
+    return feature_df
+
 def query_omim_drugbank_classifier(input_curie):
     """The main function to query the drug-disease OpenPredict classifier, 
     It queries the previously generated classifier a `.joblib` file 
@@ -405,18 +441,21 @@ def query_omim_drugbank_classifier(input_curie):
     input_namespace = parsed_curie.group(1)
     input_id = parsed_curie.group(2)
 
-    resources_folder = "data/resources/"
-    features_folder = "data/features/"
-    drugfeatfiles = ['drugs-fingerprint-sim.csv','drugs-se-sim.csv', 
-                     'drugs-ppi-sim.csv', 'drugs-target-go-sim.csv','drugs-target-seq-sim.csv']
-    diseasefeatfiles =['diseases-hpo-sim.csv',  'diseases-pheno-sim.csv' ]
-    drugfeatfiles = [ os.path.join(features_folder, fn) for fn in drugfeatfiles]
-    diseasefeatfiles = [ os.path.join(features_folder, fn) for fn in diseasefeatfiles]
+    # resources_folder = "data/resources/"
+    #features_folder = "data/features/"
+    #drugfeatfiles = ['drugs-fingerprint-sim.csv','drugs-se-sim.csv', 
+    #                 'drugs-ppi-sim.csv', 'drugs-target-go-sim.csv','drugs-target-seq-sim.csv']
+    #diseasefeatfiles =['diseases-hpo-sim.csv',  'diseases-pheno-sim.csv' ]
+    #drugfeatfiles = [ os.path.join(features_folder, fn) for fn in drugfeatfiles]
+    #diseasefeatfiles = [ os.path.join(features_folder, fn) for fn in diseasefeatfiles]
 
     ## Get all DFs
     # Merge feature matrix
-    drug_df, disease_df = mergeFeatureMatrix(drugfeatfiles, diseasefeatfiles)
-    drugDiseaseKnown = pd.read_csv(resources_folder + 'openpredict-omim-drug.csv',delimiter=',') 
+    #drug_df, disease_df = mergeFeatureMatrix(drugfeatfiles, diseasefeatfiles)
+    # (drug_df, disease_df)= load('data/features/drug_disease_dataframes.joblib')
+
+    (drug_df, disease_df)= load(pkg_resources.resource_filename('openpredict', 'data/features/drug_disease_dataframes.joblib'))
+    drugDiseaseKnown = pd.read_csv(pkg_resources.resource_filename('openpredict', 'data/resources/openpredict-omim-drug.csv'),delimiter=',') 
     drugDiseaseKnown.rename(columns={'drugid':'Drug','omimid':'Disease'}, inplace=True)
     drugDiseaseKnown.Disease = drugDiseaseKnown.Disease.astype(str)
 
@@ -431,7 +470,8 @@ def query_omim_drugbank_classifier(input_curie):
     commonDiseases=  diseaseswithfeatures.intersection(drugDiseaseKnown.Disease.unique() )
 
     # Load classifier
-    clf = load('data/models/drug_disease_model.joblib') 
+    # clf = load('data/models/drug_disease_model.joblib') 
+    clf = load(pkg_resources.resource_filename('openpredict', 'data/models/drug_disease_model.joblib')) 
 
     pairs=[]
     classes=[]
@@ -457,20 +497,7 @@ def query_omim_drugbank_classifier(input_curie):
     classes = np.array(classes)
     pairs = np.array(pairs)
 
-    
-    try:
-        logging.info('Running Spark...')
-        from pyspark import SparkConf, SparkContext
-        sc = SparkContext.getOrCreate()
-        logging.info(sc)
-        drug_df_bc= sc.broadcast(drug_df)
-        disease_df_bc = sc.broadcast(disease_df)
-        knownDrugDis_bc = sc.broadcast(pairs[classes==1])
-        test_df= sparkBuildFeatures(sc, pairs, classes, knownDrugDis_bc.value,  drug_df_bc.value, disease_df_bc.value)
-
-    except:
-        logging.info("Spark cluster not found. Using pandas to create feature dataframes")
-        test_df = createFeatureDF(pairs, classes, pairs[classes==1], drug_df, disease_df)
+    test_df = createFeaturesSparkOrDF(pairs, classes, drug_df, disease_df)
     
 
     # Get list of drug-disease pairs (should be saved somewhere from previous computer?)
@@ -484,9 +511,11 @@ def query_omim_drugbank_classifier(input_curie):
     prediction_df = pd.DataFrame( list(zip(pairs[:,0], pairs[:,1], y_proba[:,1])), columns =['drug','disease','score'])
     prediction_df.sort_values(by='score', inplace=True, ascending=False)
     # prediction_df = pd.DataFrame( list(zip(pairs[:,0], pairs[:,1], y_proba[:,1])), columns =[drug_column_label,disease_column_label,'score'])
+    
+    # Add namespace to get CURIEs from IDs
     prediction_df["drug"]= "DRUGBANK:" + prediction_df["drug"]
     prediction_df["disease"] ="OMIM:" + prediction_df["disease"]
 
-
-    prediction_results=prediction_df.to_json(orient='records')
+    # prediction_results=prediction_df.to_json(orient='records')
+    prediction_results=prediction_df.to_dict(orient='records')
     return prediction_results
